"""
Generate a CSV file with daily stats commit count from a set of repositories
"""

import datetime
import os
import logging
from collections import defaultdict
from typing import Iterable, Optional

from git import Repo

REPO = "."

logger = logging.getLogger(__name__)

def wrapper(gen: Iterable, repo: Repo):
    while True:
        try:
            yield next(gen)
        except StopIteration:
            break
        except Exception as e:
            logger.error(e)
            logger.error(f"Occured in this repo: {repo.working_dir}")
            logger.error("Will probably omit anterior commits")

def repo_daily_commits(repo: Repo, year: Optional[int] = None, author_name: Optional[str] = None):
    """Count how many commit was performed for each days in the year"""
    if author_name is not None:
        author_name = author_name.lower()
    other_author_names: set = set()
    data = defaultdict(int)
    for commit in wrapper(repo.iter_commits(all=True), repo):
        if author_name is not None:
            commit_author: str = commit.author.name
            if commit_author.lower() != author_name:
                other_author_names.add(commit_author)
        commit_timestamp: int = commit.authored_date
        commit_date: datetime.date = datetime.date.fromtimestamp(
            commit_timestamp)
        if year is not None:
            if commit_date.year != year:
                continue
        data[commit_date] += 1
    if author_name is not None:
        # Warn for all other author names
        for other_author_name in other_author_names:
            logger.info(f"Other commit author in this folder of repositories:\t{other_author_name}")
    return data

def iter_repos(dir_name: str):
    pys = []
    for root, dirs, files in os.walk(dir_name):
        for directory in dirs:
            if directory == '.git':
                yield os.path.join(root, directory)
